@@ -1,7 +1,117 @@
-jNMF <- function(X, M=NULL, initW=NULL, initV=NULL, initH=NULL, fixW=FALSE, fixV=FALSE, fixH=FALSE, J = 3, w=NULL, algorithm = c("Frobenius", "KL", "IS", "PLTF"), p=1,
+jNMF <- function(X, M=NULL, initW=NULL, initV=NULL, initH=NULL,
+    fixW=FALSE, fixV=FALSE, fixH=FALSE,
+    L1_W=1e-10, L1_V=1e-10, L1_H=1e-10,
+    L2_W=1e-10, L2_V=1e-10, L2_H=1e-10,
+    J = 3, w=NULL, algorithm = c("Frobenius", "KL", "IS", "PLTF"), p=1,
     thr = 1e-10, num.iter = 100,
     viz = FALSE, figdir = NULL, verbose = FALSE) {
     # Argument check
+    algorithm <- match.arg(algorithm)
+    chk <- .checkjNMF(X, M, initW, initV, initH, fixW, fixV, fixH, J, w,
+        p, thr, num.iter, viz, figdir, verbose)
+    X <- chk$X
+    M <- chk$M
+    fixV <- chk$fixV
+    fixH <- chk$fixH
+    w <- chk$w
+    K <- chk$K
+    # Initialization of W, V, H
+    int <- .initjNMF(X, initW, initV, initH, J, p, thr, algorithm, verbose)
+    W <- int$W
+    V <- int$V
+    H <- int$H
+    RecError <- int$RecError
+    TrainRecError <- int$TrainRecError
+    TestRecError <- int$TestRecError
+    RelChange <- int$RelChange
+    p <- int$p
+    iter <- 1
+    while ((RecError[iter] > thr) && (iter <= num.iter)) {
+        # Before Update W, H_k
+        X_bar <- lapply(seq_len(K), function(k){
+            .recMatrix(W + V[[k]], H[[k]])
+        })
+        pre_Error <- sqrt(sum(unlist(lapply(seq_along(X), function(x){
+            .recError(X[[x]], X_bar[[x]], notsqrt=TRUE)
+        }))))
+        # Update W
+        if(!fixW){
+            W_numer <- matrix(0, nrow=nrow(W), ncol=ncol(W))
+            W_denom <- matrix(0, nrow=nrow(W), ncol=ncol(W))
+            for(k in seq_len(K)){
+                W_numer <- W_numer + w[k] * (M[[k]] * X[[k]] * (M[[k]] * X_bar[[k]])^(-p)) %*% H[[k]]
+                W_denom <- W_denom + w[k] * (M[[k]] * (W + V[[k]]) %*% t(H[[k]]))^(1-p) %*% H[[k]] + L1_W + L2_W * W
+            }
+            W <- .columnNorm(.positive(W * W_numer / W_denom))
+        }
+        # Update H_k
+        for(k in seq_len(K)){
+            if(!fixH[k]){
+                Hk_numer <- (t(M[[k]] * X[[k]]) * t(M[[k]] * X_bar[[k]])^(-p)) %*% (W + V[[k]])
+                Hk_denom <- t(M[[k]] * (W + V[[k]]) %*% t(H[[k]]))^(1-p) %*% W + L1_H + L2_H * H[[k]]
+                H[[k]] <- H[[k]] * Hk_numer / Hk_denom
+            }
+        }
+        # Update V_k
+        for(k in seq_len(K)){
+            if(!fixV[k]){
+                Vk_numer <- (M[[k]] * X[[k]] * (M[[k]] * X_bar[[k]])^(-p)) %*% H[[k]]
+                Vk_denom <- (M[[k]] * (W + V[[k]]) %*% t(H[[k]]))^(1-p) %*% H[[k]] + L1_V + L2_V * V[[k]]
+                V[[k]] <- V[[k]] * Vk_numer / Vk_denom
+            }
+        }
+        # After Update W, H_k
+        iter <- iter + 1
+        X_bar <- lapply(seq_len(K), function(k){
+            .recMatrix(W + V[[k]], H[[k]])
+        })
+        RecError[iter] <- sqrt(sum(unlist(lapply(seq_along(X), function(x){
+            .recError(X[[x]], X_bar[[x]], notsqrt=TRUE)
+        }))))
+        TrainRecError[iter] <- sqrt(sum(unlist(lapply(seq_along(X), function(x){
+            .recError(M[[x]]*X[[x]], M[[x]]*X_bar[[x]], notsqrt=TRUE)
+        }))))
+        TestRecError[iter] <- sqrt(sum(unlist(lapply(seq_along(X), function(x){
+            .recError((1-M[[x]])*X[[x]], (1-M[[x]])*X_bar[[x]], notsqrt=TRUE)
+        }))))
+        RelChange[iter] <- abs(pre_Error - RecError[iter]) / RecError[iter]
+        if (viz && !is.null(figdir)) {
+            png(filename = paste0(figdir, "/", iter-1, ".png"))
+            .imageplot_jNMF(X, W, V, H)
+            dev.off()
+        }
+        if (viz && is.null(figdir)) {
+            .imageplot_jNMF(X, W, V, H)
+        }
+        if (verbose) {
+            cat(paste0(iter-1, " / ", num.iter, " |Previous Error - Error| / Error = ",
+                RelChange[iter], "\n"))
+        }
+        if (is.nan(RelChange[iter])) {
+            stop("NaN is generated. Please run again or change the parameters.\n")
+        }
+    }
+    if (viz && !is.null(figdir)) {
+        png(filename = paste0(figdir, "/finish.png"))
+        .imageplot_jNMF(X, W, V, H)
+        dev.off()
+    }
+    if (viz && is.null(figdir)) {
+        .imageplot_jNMF(X, W, V, H)
+    }
+    names(RecError) <- c("offset", 1:(iter-1))
+    names(TrainRecError) <- c("offset", 1:(iter-1))
+    names(TestRecError) <- c("offset", 1:(iter-1))
+    names(RelChange) <- c("offset", 1:(iter-1))
+    return(list(W = W, V = V, H = H,
+        RecError = RecError,
+        TrainRecError = TrainRecError,
+        TestRecError = TestRecError,
+        RelChange = RelChange))
+}
+
+.checkjNMF <- function(X, M, initW, initV, initH, fixW, fixV, fixH, J, w,
+    p, thr, num.iter, viz, figdir, verbose){
     if(!is.list(X)){
         stop("input X must be specified as a list!")
     }
@@ -76,7 +186,6 @@ jNMF <- function(X, M=NULL, initW=NULL, initV=NULL, initH=NULL, fixW=FALSE, fixV
             w <- w / sum(w)
         }
     }
-    algorithm <- match.arg(algorithm)
     if(!is.numeric(p)){
         stop("Please specify the p as a number")
     }
@@ -95,18 +204,15 @@ jNMF <- function(X, M=NULL, initW=NULL, initV=NULL, initH=NULL, fixW=FALSE, fixV
     if(!is.logical(verbose)){
         stop("Please specify the verbose as a logical")
     }
-
     K <- length(X)
-    lapply(seq_along(X), function(x){
-        X[[x]][which(X[[x]] == 0)] <<- 1e-10
-        M[[x]][which(M[[x]] == 0)] <<- 1e-10
-    })
+    list(X=X, M=M, fixV=fixV, fixH=fixH, w=w, K=K)
+}
 
-    # Initialization of W, V, H
+.initjNMF <- function(X, initW, initV, initH, J, p, thr, algorithm, verbose){
     if(is.null(initW)){
         W <- matrix(runif(nrow(X[[1]])*J),
             nrow=nrow(X[[1]]), ncol=J)
-        W <- .columnNorm(W * W)        
+        W <- .columnNorm(W * W)
     }else{
         W <- initW
     }
@@ -115,7 +221,7 @@ jNMF <- function(X, M=NULL, initW=NULL, initV=NULL, initH=NULL, fixW=FALSE, fixV
             tmp <- matrix(runif(nrow(X[[1]])*J),
             nrow=nrow(X[[1]]), ncol=J)
             .columnNorm(tmp * tmp)
-        })        
+        })
     }else{
         V <- initV
     }
@@ -124,22 +230,18 @@ jNMF <- function(X, M=NULL, initW=NULL, initV=NULL, initH=NULL, fixW=FALSE, fixV
             tmp <- matrix(runif(ncol(X[[x]])*J),
                 nrow=ncol(X[[x]]), ncol=J)
             .columnNorm(tmp * tmp)
-        })        
+        })
     }else{
         H <- initH
     }
-
     RecError = c()
     TrainRecError = c()
     TestRecError = c()
     RelChange = c()
-
     RecError[1] <- thr * 10
     TrainRecError[1] <- thr * 10
     TestRecError[1] <- thr * 10
     RelChange[1] <- thr * 10
-
-    iter <- 1
     # Algorithm
     if (algorithm == "Frobenius") {
         p = 0
@@ -156,95 +258,7 @@ jNMF <- function(X, M=NULL, initW=NULL, initV=NULL, initH=NULL, fixW=FALSE, fixV
     if (verbose) {
         cat("Iterative step is running...\n")
     }
-    while ((RecError[iter] > thr) && (iter <= num.iter)) {
-        # Before Update W, H_k
-        X_bar <- lapply(seq_len(K), function(k){
-            .recMatrix(W + V[[k]], H[[k]])
-        })
-        pre_Error <- sqrt(sum(unlist(lapply(seq_along(X), function(x){
-            .recError(X[[x]], X_bar[[x]], notsqrt=TRUE)
-        }))))
-
-        # Update W
-        if(!fixW){
-            W_numer <- matrix(0, nrow=nrow(W), ncol=ncol(W))
-            W_denom <- matrix(0, nrow=nrow(W), ncol=ncol(W))
-            for(k in seq_len(K)){
-                W_numer <- W_numer + w[k] * (M[[k]] * X[[k]] * (M[[k]] * X_bar[[k]])^(-p)) %*% H[[k]]
-                W_denom <- W_denom + w[k] * (M[[k]] * (W + V[[k]]) %*% t(H[[k]]))^(1-p) %*% H[[k]]
-            }
-            W <- .columnNorm(.positive(W * W_numer / W_denom))
-        }
-
-        # Update H_k        
-        for(k in seq_len(K)){
-            if(!fixH[k]){                
-                Hk_numer <- (t(M[[k]] * X[[k]]) * t(M[[k]] * X_bar[[k]])^(-p)) %*% (W + V[[k]])
-                Hk_denom <- t(M[[k]] * (W + V[[k]]) %*% t(H[[k]]))^(1-p) %*% W
-                H[[k]] <- H[[k]] * Hk_numer / Hk_denom
-            }
-        }
-
-        # Update V_k
-        for(k in seq_len(K)){
-            if(!fixV[k]){                
-                Vk_numer <- (M[[k]] * X[[k]] * (M[[k]] * X_bar[[k]])^(-p)) %*% H[[k]]
-                Vk_denom <- (M[[k]] * (W + V[[k]]) %*% t(H[[k]]))^(1-p) %*% H[[k]]
-                V[[k]] <- V[[k]] * Vk_numer / Vk_denom
-            }
-        }
-        # After Update W, H_k
-        iter <- iter + 1
-        X_bar <- lapply(seq_len(K), function(k){
-            .recMatrix(W + V[[k]], H[[k]])
-        })
-        RecError[iter] <- sqrt(sum(unlist(lapply(seq_along(X), function(x){
-            .recError(X[[x]], X_bar[[x]], notsqrt=TRUE)
-        }))))
-
-        TrainRecError[iter] <- sqrt(sum(unlist(lapply(seq_along(X), function(x){
-            .recError(M[[x]]*X[[x]], M[[x]]*X_bar[[x]], notsqrt=TRUE)
-        }))))
-
-        TestRecError[iter] <- sqrt(sum(unlist(lapply(seq_along(X), function(x){
-            .recError((1-M[[x]])*X[[x]], (1-M[[x]])*X_bar[[x]], notsqrt=TRUE)
-        }))))
-
-        RelChange[iter] <- abs(pre_Error - RecError[iter]) / RecError[iter]
-
-        if (viz && !is.null(figdir)) {
-            png(filename = paste0(figdir, "/", iter-1, ".png"))
-            .imageplot_jNMF(X, W, V, H)
-            dev.off()
-        }
-        if (viz && is.null(figdir)) {
-            .imageplot_jNMF(X, W, V, H)
-        }
-        if (verbose) {
-            cat(paste0(iter-1, " / ", num.iter, " |Previous Error - Error| / Error = ",
-                RelChange[iter], "\n"))
-        }
-        if (is.nan(RelChange[iter])) {
-            stop("NaN is generated. Please run again or change the parameters.\n")
-        }
-    }
-    if (viz && !is.null(figdir)) {
-        png(filename = paste0(figdir, "/finish.png"))
-        .imageplot_jNMF(X, W, V, H)
-        dev.off()
-    }
-    if (viz && is.null(figdir)) {
-        .imageplot_jNMF(X, W, V, H)
-    }
-    names(RecError) <- c("offset", 1:(iter-1))
-    names(TrainRecError) <- c("offset", 1:(iter-1))
-    names(TestRecError) <- c("offset", 1:(iter-1))
-    names(RelChange) <- c("offset", 1:(iter-1))
-    return(list(W = W, V = V, H = H,
-        RecError = RecError,
-        TrainRecError = TrainRecError,
-        TestRecError = TestRecError,
-        RelChange = RelChange))
+    list(W=W, V=V, H=H, RecError=RecError, TrainRecError=TrainRecError, TestRecError=TestRecError, RelChange=RelChange, p=p)
 }
 
 .imageplot_jNMF <- function(X, W, V, H){
